# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

from Game import GameState
import GridUtils

DIRECTIONS = ((1,1),(1,-1),(1,0),(0,1))

SCORES = {3: 1, 6: 2, 9: 3}

def score_for_position(grid, pos):
    # only 2d for now
    total = 0
    # count total number of pieces in each direction (including new piece) and score for 3/6/9
    for d in DIRECTIONS:
        num_in_line = 1
        num_in_line += sum(1 for p,val in GridUtils.enumerate_values_with_direction(grid, pos, d) if val!=0)
        num_in_line += sum(1 for p,val in GridUtils.enumerate_values_with_direction(grid, pos, GridUtils.negate_tuple(d)) if val!=0)
        total += SCORES.get(num_in_line, 0)
    return total

class ThreeSixNineGameState(GameState):
    def reset(self):
        GameState.reset(self)
        self.extra_state = {'player_scores': [0,0]}

    def player_score(self, pnum):
        return self.extra_state['player_scores'][pnum-1]
        
    def copy_extra_state(self):
        return {'player_scores': self.extra_state['player_scores'][:]}

    def successors(self):
        for c in self.enumerate_coords_with_value(0):
            nextgrid = self.grid.copy()
            nextgrid[c] = self.current_player
            nextstate = self.next_state_with_grid(nextgrid)
            # add score for this move
            nextstate.extra_state['player_scores'][self.current_player-1] += score_for_position(nextgrid, c)
            yield ( (c,), nextstate )
